Package: eSDM
Title: Ensemble Tool for Predictions from Species Distribution Models
Description: A tool which allows users to create and evaluate ensembles 
    of species distribution model (SDM) predictions. 
    Functionality is offered through R functions or a GUI (R Shiny app). 
    This tool can assist users in identifying spatial uncertainties and 
    making informed conservation and management decisions. The package is 
    further described in Woodman et al (2019) <doi:10.1111/2041-210X.13283>.
Version: 0.4.1
Authors@R: person("Sam", "Woodman", email = "sam.woodman@noaa.gov", role = c("aut", "cre"), comment = c(ORCID = "0000-0001-6071-8186"))
URL: https://github.com/smwoodman/eSDM/,
        https://connect.fisheries.noaa.gov/eSDM/
BugReports: https://github.com/smwoodman/eSDM/issues/
Depends: R (>= 4.0.0)
Imports: dplyr (>= 0.7-0), magrittr, methods, purrr, rlang, ROCR, sf
        (>= 0.9-0), shiny, stats, units
Suggests: colorRamps, colourpicker, dichromat, DT, knitr, leafem,
        leaflet, maps, raster, RColorBrewer, rmarkdown, shinybusy,
        shinydashboard, shinyjs, testthat (>= 2.1.0), tmap (>= 2.3),
        viridis, zip
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.1
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2024-02-14 20:15:05 UTC; sam.woodman
Author: Sam Woodman [aut, cre] (<https://orcid.org/0000-0001-6071-8186>)
Maintainer: Sam Woodman <sam.woodman@noaa.gov>
Repository: CRAN
Date/Publication: 2024-02-14 23:30:02 UTC
Built: R 4.2.3; ; 2024-02-18 06:30:43 UTC; unix
