# Generated by roxygen2: do not edit by hand

S3method(ensemble_create,data.frame)
S3method(ensemble_create,sf)
export(eSDM_GUI)
export(ensemble_create)
export(ensemble_rescale)
export(evaluation_metrics)
export(model_abundance)
export(overlay_sdm)
export(pts2poly_centroids)
export(pts2poly_vertices)
importFrom(ROCR,performance)
importFrom(ROCR,prediction)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,between)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(magrittr,"%>%")
importFrom(methods,slot)
importFrom(purrr,map)
importFrom(purrr,map2_df)
importFrom(purrr,set_names)
importFrom(rlang,.data)
importFrom(rlang,sym)
importFrom(sf,st_agr)
importFrom(sf,st_area)
importFrom(sf,st_bbox)
importFrom(sf,st_crop)
importFrom(sf,st_crs)
importFrom(sf,st_geometry)
importFrom(sf,st_intersection)
importFrom(sf,st_intersects)
importFrom(sf,st_is_valid)
importFrom(sf,st_make_valid)
importFrom(sf,st_polygon)
importFrom(sf,st_set_agr)
importFrom(sf,st_set_geometry)
importFrom(sf,st_sf)
importFrom(sf,st_sfc)
importFrom(shiny,runApp)
importFrom(stats,na.omit)
importFrom(units,set_units)
importFrom(utils,head)
importFrom(utils,tail)
