This folder contains the data used in the 'Example analysis' section of Woodman et al. (in review), as well as the 'example-analysis' vignette in the eSDM package. This data is intended to be used to reproduce the results of the example analysis, or by users familiarizing themselves with the file formats required by and functionality of the eSDM GUI. If you use any of this data in your own analyses, please first consult with the author of the relevant data source, and then cite the data (see references section for citation information).


-------------------------------------------------------------------------------
PREDICTION DATA
Description: Several sets of predictions from species distribution models (SDM) of blue whale occurrence off the U.S. West Coast from August-November. The predictions are the mean of predictions from 2001, 2005, and 2008 for each model. The standard error (SE) values for the predictions were calculated using the interannual variability, following the methods of Becker et al. (2016).

Predictions_Beckeretal2016.csv
Description: An Excel .csv file with SDM predictions of absolute density from Becker et al. (2016). Predictions were made on a grid with cells of size 0.09 by 0.09 WGS 84 decimal degrees. This .csv file contains columns with the longitude and latitude coordinates of the centroids of those cells ("lon" and "lat", respectively), the mean prediction values ("pred_bm"), and associated SE values ("se"). Please contact Elizabeth Becker (ebecker77@cox.net) and cite Becker et al. (2016) if using these data.

Predictions_Hazenetal2017.csv
Description: An Excel .csv file with SDM predictions of habitat likelihood from Hazen et al. (2017). Predictions were made on a grid with cells of size 0.25 by 0.25 WGS 84 decimal degrees. This .csv file contains columns with the longitude and latitude coordinates of the centroids of those cells ("lon" and "lat", respectively), the mean prediction values ("pred_bm"), and associated SE values ("se"). Please contact Elliott Hazen (elliott.hazen@noaa.gov) and cite Hazen et al. (2017) if using these data.

Predictions_Redfernetal2017 (Files: Predictions_Redfernetal2017.dbf, Predictions_Redfernetal2017.prj, Predictions_Redfernetal2017.shp, Predictions_Redfernetal2017.shx)
Description: A GIS shapefile with SDM predictions of relative density from Redfern et al. (2017). Predictions were made on a grid with cells of size 10 by 10 kilometers. The predictions are in a cylindrical equal area coordinate system. The shapefile contains the data columns with the mean prediction values ("pred_bm") and associated SE values ("se"). Please contact Jessica Redfern (jessica.redfern@noaa.gov) and cite Redfern et al. (2017) if using these data.


-------------------------------------------------------------------------------
VALIDATION DATA
Description: Presence/absence (binary) validation data from Irvine et al. (2014) and Becker et al. (2016) used to calculate AUC and TSS.

eSDM_Validation_data_all.csv
Description: A .csv file of blue whale presence/absence points generated from both blue whale home range data (Irvine et al. 2014) and line transect segments (Becker et al. 2016). See Becker et al. (2016) and Woodman et al. (in review), respectively, for more information on how these points were generated from these two data sets. This .csv file contains columns with longitude and latitude coordinates of the points ("lon" and "lat", respectively), the source of the data point ("source"), and a blue whale presence/absence indicator, i.e. a validation data column ("pres_abs"). The "source" column consists of two values, "Irvine_et_al_2014” for home range data and "Becker_et_al_2016" for line transect segments. The presence/absence indicator column consists of the numbers “0” and “1”, with “0” indicating absence and “1” indicating presence. This validation data is specific to the example analysis in Woodman et al. (in review) and should not be used without consultation with the authors. Please cite Irvine et al. (2014), Becker et al. (2016), and Woodman et al. (in review) if using these data.


-------------------------------------------------------------------------------
OTHER FILES
Description: These files are specific to the example analysis in Woodman et al. (in prep) and should not be used without consultation with the authors.

Study_Area_CCE (Files: Study_Area_CCE.dbf, Study_Area_CCE.prj, Study_Area_CCE.sbn, Study_Area_CCE.sbx, Study_Area_CCE.shp, Study_Area_CCE.shx)
Description: A GIS shapefile with the study area boundary (based on the California Current Ecosystem; CCE) generated by the Southwest Fisheries Science Center's Marine Mammal and Turtle Division (previously Protected Resources Division).


-------------------------------------------------------------------------------
REFERENCES

Becker, E., Forney, K., Fiedler, P., Barlow, J., Chivers, S., Edwards, C., ... Redfern, J. (2016) Moving towards dynamic ocean management: how well do modeled ocean products predict species distributions? Remote Sensing, 8, 149. 10.3390/rs8020149

Hazen, E.L., Palacios, D.M., Forney, K.A., Howell, E.A., Becker, E., Hoover, A.L., ... Bailey, H. (2017) WhaleWatch: a dynamic management tool for predicting blue whale density in the California Current. Journal of Applied Ecology, 54, 1415-1428. 10.1111/1365-2664.12820

Irvine, L.M., Mate, B.R., Winsor, M.H., Palacios, D.M., Bograd, S.J., Costa, D.P. & Bailey, H. (2014) Spatial and temporal occurrence of blue whales off the U.S. West Coast, with implications for management. PLoS One, 9, e102959. 10.1371/journal.pone.0102959

Redfern, J.V., Moore, T.J., Fiedler, P.C., de Vos, A., Brownell, R.L., Forney, K.A., ... Heikkinen, R. (2017) Predicting cetacean distributions in data-poor marine ecosystems. Diversity and Distributions, 23, 394-408. 10.1111/ddi.12537

Woodman, S.M., Forney, K.A., Becker, E.A., DeAngelis, M.L., Hazen, E.L., Palacios, D.M., Redfern, J.V. (in review) eSDM: A tool for creating and exploring ensembles of predictions from species distribution models.
